@@ -1,4 +1,6 @@
+import copy
 import warnings
+import patsy
 import numpy as np
 import pandas as pd
 import matplotlib.pyplot as plt
@@ -7,9 +9,11 @@ import statsmodels.formula.api as smf
 from statsmodels.stats.weightstats import DescrStatsW
 from scipy.stats import norm
 
-from zepid.causal.utils import (propensity_score, plot_kde, plot_love, iptw_calculator,
-                                standardized_mean_differences, positivity, _bounding_,
-                                plot_kde_accuracy, outcome_accuracy)
+from zepid.calc import probability_bounds
+from zepid.causal.utils import (check_input_data, propensity_score, plot_kde, plot_love, iptw_calculator,
+                                standardized_mean_differences, positivity,
+                                plot_kde_accuracy, outcome_accuracy, aipw_calculator,
+                                exposure_machine_learner, outcome_machine_learner, missing_machine_learner)
 
 
 class AIPTW:
@@ -105,38 +109,24 @@ class AIPTW:
 
     References
     ----------
-    Funk, M. J., Westreich, D., Wiesen, C., Stürmer, T., Brookhart, M. A., & Davidian, M. (2011). Doubly robust
+    Funk MJ, Westreich D, Wiesen C, Stürmer T, Brookhart MA, & Davidian M. (2011). Doubly robust
     estimation of causal effects. American Journal of Epidemiology, 173(7), 761-767.
 
     Lunceford JK, Davidian M. (2004). Stratification and weighting via the propensity score in estimation of causal
     treatment effects: a comparative study. Statistics in medicine, 23(19), 2937-2960.
     """
+
     def __init__(self, df, exposure, outcome, weights=None, alpha=0.05):
-        if df.dropna(subset=[d for d in df.columns if d != outcome]).shape[0] != df.shape[0]:
-            warnings.warn("There is missing data that is not the outcome in the data set. AIPTW will drop "
-                          "all missing data that is not missing outcome data. AIPTW will fit "
-                          + str(df.dropna(subset=[d for d in df.columns if d != outcome]).shape[0]) +
-                          ' of ' + str(df.shape[0]) + ' observations', UserWarning)
-            self.df = df.copy().dropna(subset=[d for d in df.columns if d != outcome]).reset_index()
-        else:
-            self.df = df.copy().reset_index()
-
-        # Checking to see if missing outcome data occurs
-        self._missing_indicator = '__missing_indicator__'
-        if self.df.dropna(subset=[outcome]).shape[0] != self.df.shape[0]:
-            self._miss_flag = True
-            self.df[self._missing_indicator] = np.where(self.df[outcome].isna(), 0, 1)
-        else:
-            self._miss_flag = False
-            self.df[self._missing_indicator] = 1
-
         self.exposure = exposure
         self.outcome = outcome
-
-        if df[outcome].dropna().value_counts().index.isin([0, 1]).all():
-            self._continuous_outcome = False
-        else:
-            self._continuous_outcome = True
+        self._missing_indicator = '__missing_indicator__'
+        self.df, self._miss_flag, self._continuous_outcome = check_input_data(data=df,
+                                                                              exposure=exposure,
+                                                                              outcome=outcome,
+                                                                              estimator="AIPTW",
+                                                                              drop_censoring=False,
+                                                                              drop_missing=True,
+                                                                              binary_exposure_only=True)
 
         self._weight_ = weights
         self.alpha = alpha
@@ -146,18 +136,24 @@ class AIPTW:
         self.risk_difference_ci = None
         self.risk_ratio_ci = None
         self.risk_difference_se = None
+        self.risk_ratio_se = None
 
         self.average_treatment_effect = None
         self.average_treatment_effect_ci = None
         self.average_treatment_effect_se = None
 
+        self._continuous_type = None
         self._fit_exposure_ = False
+        self._exp_model_custom = False
         self._fit_outcome_ = False
+        self._out_model_custom = False
         self._fit_missing_ = False
+        self._miss_model_custom = False
         self._exp_model = None
         self._out_model = None
+        self._predicted_y_ = None
 
-    def exposure_model(self, model, bound=False, print_results=True):
+    def exposure_model(self, model, custom_model=None, bound=False, print_results=True):
         r"""Specify the propensity score / inverse probability weight model. Model used to predict the exposure via a
         logistic regression model. This model estimates
 
@@ -169,6 +165,10 @@ class AIPTW:
         ----------
         model : str
             Independent variables to predict the exposure. For example, 'var1 + var2 + var3'
+        custom_model : optional
+            Input for a custom model that is used in place of the logit model (default). The model must have the
+            "fit()" and  "predict()" attributes. SciKit-Learn style models supported as custom models. In the
+            background, AIPTW will fit the custom model and generate the predicted probablities
         bound : float, list, optional
             Value between 0,1 to truncate predicted probabilities. Helps to avoid near positivity violations.
             Specifying this argument can improve finite sample performance for random positivity violations. However,
@@ -181,20 +181,32 @@ class AIPTW:
         """
         self.__mweight = model
         self._exp_model = self.exposure + ' ~ ' + model
-        d, n, iptw = iptw_calculator(df=self.df, treatment=self.exposure, model_denom=model, model_numer='1',
-                                     weight=self._weight_, stabilized=False, standardize='population',
-                                     bound=None, print_results=print_results)
 
-        self.df['_g1_'] = d
-        self.df['_g0_'] = 1 - d
+        if custom_model is None:
+            d, n, iptw = iptw_calculator(df=self.df, treatment=self.exposure, model_denom=model, model_numer='1',
+                                         weight=self._weight_, stabilized=False, standardize='population',
+                                         bound=None, print_results=print_results)
+        else:
+            self._exp_model_custom = True
+            data = patsy.dmatrix(model + ' - 1', self.df)
+            d = exposure_machine_learner(xdata=np.asarray(data),
+                                         ydata=np.asarray(self.df[self.exposure]),
+                                         ml_model=copy.deepcopy(custom_model),
+                                         print_results=print_results)
+
+        g1w = d
+        g0w = 1 - d
+
         # Applying bounds AFTER extracting g1 and g0
         if bound:
-            self.df['_g1_'] = _bounding_(self.df['_g1_'], bounds=bound)
-            self.df['_g0_'] = _bounding_(self.df['_g0_'], bounds=bound)
+            g1w = probability_bounds(g1w, bounds=bound)
+            g0w = probability_bounds(g0w, bounds=bound)
 
+        self.df['_g1_'] = g1w
+        self.df['_g0_'] = g0w
         self._fit_exposure_ = True
 
-    def missing_model(self, model, bound=False, print_results=True):
+    def missing_model(self, model, custom_model=None, bound=False, print_results=True):
         r"""Estimation of Pr(M=0|A,L), which is the missing data mechanism for the outcome. Predicted probabilities are
         used to create inverse probability of censoring weights to account for informative missing data on the outcome.
 
@@ -214,6 +226,10 @@ class AIPTW:
         model : str
             Independent variables to predict the exposure. Example) 'var1 + var2 + var3'. The treatment must be
             included for the missing data model
+        custom_model : optional
+            Input for a custom model that is used in place of the logit model (default). The model must have the
+            "fit()" and  "predict()" attributes. SciKit-Learn style models are supported as custom models. In the
+            background, AIPTW will fit the custom model and generate the predicted probablities
         bound : float, list, optional
             Value between 0,1 to truncate predicted probabilities. Helps to avoid near positivity violations.
             Specifying this argument can improve finite sample performance for random positivity violations. However,
@@ -241,23 +257,40 @@ class AIPTW:
         self._miss_model = self._missing_indicator + ' ~ ' + model
         fitmodel = propensity_score(self.df, self._miss_model, print_results=print_results)
 
-        dfx = self.df.copy()
-        dfx[self.exposure] = 1
-        self.df['_ipmw_a1_'] = np.where(self.df[self._missing_indicator] == 1,
-                                        fitmodel.predict(dfx), np.nan)
-        dfx = self.df.copy()
-        dfx[self.exposure] = 0
-        self.df['_ipmw_a0_'] = np.where(self.df[self._missing_indicator] == 1,
-                                        fitmodel.predict(dfx), np.nan)
+        if custom_model is None:  # Logistic Regression model for predictions
+            dfx = self.df.copy()
+            dfx[self.exposure] = 1
+            m1w = np.where(self.df[self._missing_indicator] == 1, fitmodel.predict(dfx), np.nan)
+            dfx = self.df.copy()
+            dfx[self.exposure] = 0
+            m0w = np.where(self.df[self._missing_indicator] == 1, fitmodel.predict(dfx), np.nan)
+        else:  # User-Specified model
+            self._miss_model_custom = True
+            data = patsy.dmatrix(model + ' - 1', self.df)
+            dfx = self.df.copy()
+            dfx[self.exposure] = 1
+            adata = patsy.dmatrix(model + ' - 1', dfx)
+            dfx = self.df.copy()
+            dfx[self.exposure] = 0
+            ndata = patsy.dmatrix(model + ' - 1', dfx)
+
+            m1w, m0w = missing_machine_learner(xdata=np.array(data),
+                                               mdata=self.df[self._missing_indicator],
+                                               all_a=adata, none_a=ndata,
+                                               ml_model=copy.deepcopy(custom_model),
+                                               print_results=print_results)
 
         # If bounds are requested
         if bound:
-            self.df['_ipmw_a1_'] = _bounding_(self.df['_ipmw_a1_'], bounds=bound)
-            self.df['_ipmw_a0_'] = _bounding_(self.df['_ipmw_a0_'], bounds=bound)
+            m1w = probability_bounds(m1w, bounds=bound)
+            m0w = probability_bounds(m0w, bounds=bound)
+
+        self.df['_ipmw_a1_'] = m1w
+        self.df['_ipmw_a0_'] = m0w
 
         self._fit_missing_ = True
 
-    def outcome_model(self, model, continuous_distribution='gaussian', print_results=True):
+    def outcome_model(self, model, custom_model=None, continuous_distribution='gaussian', print_results=True):
         r"""Specify the outcome model. Model used to predict the outcome via a regression model. For binary outcome
         data, a logistic regression model is used. For continuous outcomes, either linear or Poisson regression are
         available.
@@ -270,6 +303,10 @@ class AIPTW:
         ----------
         model : str
             Independent variables to predict the outcome. For example, 'var1 + var2 + var3 + var4'
+        custom_model : optional
+            Input for a custom model that is used in place of the logit model (default). The model must have the
+            "fit()" and  "predict()" attributes. SciKit-Learn style models are supported as custom models. In the
+            background, TMLE will fit the custom model and generate the predicted values
         continuous_distribution : str, optional
             Distribution to use for continuous outcomes. Options are 'gaussian' for normal distributions and 'poisson'
             for Poisson distributions
@@ -279,40 +316,65 @@ class AIPTW:
         if self.exposure not in model:
             warnings.warn("It looks like '" + self.exposure + "' is not included in the outcome model.")
 
+        if self._miss_flag:
+            cc = self.df.copy().dropna()
+        else:
+            cc = self.df.copy()
+
         self._out_model = self.outcome + ' ~ ' + model
 
-        if self._continuous_outcome:
-            self._continuous_type = continuous_distribution
-            if (continuous_distribution == 'gaussian') or (continuous_distribution == 'normal'):
-                f = sm.families.family.Gaussian()
-            elif continuous_distribution == 'poisson':
-                f = sm.families.family.Poisson()
+        if custom_model is None:
+            if self._continuous_outcome:
+                self._continuous_type = continuous_distribution
+                if (continuous_distribution == 'gaussian') or (continuous_distribution == 'normal'):
+                    f = sm.families.family.Gaussian()
+                elif continuous_distribution == 'poisson':
+                    f = sm.families.family.Poisson()
+                else:
+                    raise ValueError("Only 'gaussian' and 'poisson' distributions are supported")
             else:
-                raise ValueError("Only 'gaussian' and 'poisson' distributions are supported")
+                f = sm.families.family.Binomial()
+
+            if self._weight_ is None:
+                log = smf.glm(self._out_model, cc, family=f).fit()
+            else:
+                log = smf.glm(self._out_model, cc, freq_weights=cc[self._weight_], family=f).fit()
+
+            if print_results:
+                print('==============================================================================')
+                print('Outcome Model')
+                print(log.summary())
+                print('==============================================================================')
+
+            # Predicting under treatment strategies
+            dfx = self.df.copy()
+            dfx[self.exposure] = 1
+            qa1w = log.predict(dfx)
+            dfx = self.df.copy()
+            dfx[self.exposure] = 0
+            qa0w = log.predict(dfx)
+
         else:
-            f = sm.families.family.Binomial()
+            self._out_model_custom = True
+            data = patsy.dmatrix(model + ' - 1', cc)
 
-        if self._weight_ is None:
-            log = smf.glm(self._out_model, self.df, family=f).fit()
-        else:
-            log = smf.glm(self._out_model, self.df, freq_weights=self.df[self._weight_], family=f).fit()
+            dfx = self.df.copy()
+            dfx[self.exposure] = 1
+            adata = patsy.dmatrix(model + ' - 1', dfx)
+            dfx = self.df.copy()
+            dfx[self.exposure] = 0
+            ndata = patsy.dmatrix(model + ' - 1', dfx)
 
-        if print_results:
-            print('\n----------------------------------------------------------------')
-            print('MODEL: ' + self._out_model)
-            print('-----------------------------------------------------------------')
-            print(log.summary())
+            qa1w, qa0w = outcome_machine_learner(xdata=np.asarray(data),
+                                                 ydata=np.asarray(cc[self.outcome]),
+                                                 all_a=adata, none_a=ndata,
+                                                 ml_model=copy.deepcopy(custom_model),
+                                                 continuous=self._continuous_outcome,
+                                                 print_results=print_results)
 
-        # Generating predictions for observed variables
-        self._predicted_y_ = log.predict(self.df)
-
-        # Predicting under treatment strategies
-        dfx = self.df.copy()
-        dfx[self.exposure] = 1
-        self.df['_pY1_'] = log.predict(dfx)
-        dfx = self.df.copy()
-        dfx[self.exposure] = 0
-        self.df['_pY0_'] = log.predict(dfx)
+        self.df['_pY1_'] = qa1w
+        self.df['_pY0_'] = qa0w
+        self._predicted_y_ = qa1w * self.df[self.exposure] + qa0w * (1 - self.df[self.exposure])
         self._fit_outcome_ = True
 
     def fit(self):
@@ -338,55 +400,52 @@ class AIPTW:
                           "function", UserWarning)
 
         # Doubly robust estimator under all treated
-        a_obs = self.df[self.exposure]
-        y_obs = self.df[self.outcome]
-        py_a1 = self.df['_pY1_']
-        py_a0 = self.df['_pY0_']
+        a_obs = np.asarray(self.df[self.exposure])
+        y_obs = np.asarray(self.df[self.outcome])
+        py_a1 = np.asarray(self.df['_pY1_'])
+        py_a0 = np.asarray(self.df['_pY0_'])
 
         if self._fit_missing_:
-            ps_g1 = self.df['_g1_'] * self.df['_ipmw_a1_']
-            ps_g0 = self.df['_g0_'] * self.df['_ipmw_a0_']
+            ps_g1 = np.asarray(self.df['_g1_'] * self.df['_ipmw_a1_'])
+            ps_g0 = np.asarray(self.df['_g0_'] * self.df['_ipmw_a0_'])
         else:
-            ps_g1 = self.df['_g1_']
-            ps_g0 = self.df['_g0_']
+            ps_g1 = np.asarray(self.df['_g1_'])
+            ps_g0 = np.asarray(self.df['_g0_'])
 
-        # Doubly robust estimator under all treated
-        dr_a1 = np.where(a_obs == 1,
-                         (y_obs / ps_g1) - ((py_a1 * ps_g0) / ps_g1),
-                         py_a1)
-
-        # Doubly robust estimator under all untreated
-        dr_a0 = np.where(a_obs == 1,
-                         py_a0,
-                         (y_obs / ps_g0 - ((py_a0 * ps_g1) / ps_g0)))
+        if self._weight_ is None:
+            w = None
+        else:
+            w = self.df[self._weight_]
+        diff_est, diff_var = aipw_calculator(y=y_obs, a=a_obs,
+                                             py_a=py_a1, py_n=py_a0,
+                                             pa1=ps_g1, pa0=ps_g0,
+                                             difference=True, weights=w,
+                                             splits=None, continuous=self._continuous_outcome)
 
         # Generating estimates for the risk difference and risk ratio
         zalpha = norm.ppf(1 - self.alpha / 2, loc=0, scale=1)
 
-        if self._weight_ is None:
-            if self._continuous_outcome:
-                self.average_treatment_effect = np.nanmean(dr_a1) - np.nanmean(dr_a0)
-                var_ic = np.nanvar((dr_a1 - dr_a0) - self.average_treatment_effect, ddof=1) / self.df.shape[0]
-                self.average_treatment_effect_se = np.sqrt(var_ic)
-                self.average_treatment_effect_ci = [self.average_treatment_effect - zalpha * np.sqrt(var_ic),
-                                                    self.average_treatment_effect + zalpha * np.sqrt(var_ic)]
+        if self._continuous_outcome:
+            self.average_treatment_effect = diff_est
+            self.average_treatment_effect_se = np.sqrt(diff_var)
+            self.average_treatment_effect_ci = [self.average_treatment_effect - zalpha * np.sqrt(diff_var),
+                                                self.average_treatment_effect + zalpha * np.sqrt(diff_var)]
 
-            else:
-                self.risk_difference = np.nanmean(dr_a1) - np.nanmean(dr_a0)
-                self.risk_ratio = np.nanmean(dr_a1) / np.nanmean(dr_a0)
-                var_ic = np.nanvar((dr_a1 - dr_a0) - self.risk_difference, ddof=1) / self.df.shape[0]
-                self.risk_difference_se = np.sqrt(var_ic)
-                self.risk_difference_ci = [self.risk_difference - zalpha * np.sqrt(var_ic),
-                                           self.risk_difference + zalpha * np.sqrt(var_ic)]
         else:
-            dr_m1 = DescrStatsW(dr_a1, weights=self.df[self._weight_]).mean
-            dr_m0 = DescrStatsW(dr_a0, weights=self.df[self._weight_]).mean
+            self.risk_difference = diff_est
+            self.risk_difference_se = np.sqrt(diff_var)
+            self.risk_difference_ci = [self.risk_difference - zalpha * np.sqrt(diff_var),
+                                       self.risk_difference + zalpha * np.sqrt(diff_var)]
 
-            if self._continuous_outcome:
-                self.average_treatment_effect = dr_m1 - dr_m0
-            else:
-                self.risk_difference = dr_m1 - dr_m0
-                self.risk_ratio = dr_m1 / dr_m0
+            rr, ln_rr_var = aipw_calculator(y=y_obs, a=a_obs,
+                                            py_a=py_a1, py_n=py_a0,
+                                            pa1=ps_g1, pa0=ps_g0,
+                                            difference=False, weights=w,
+                                            splits=None, continuous=False)
+            self.risk_ratio = rr
+            self.risk_ratio_se = np.sqrt(ln_rr_var)
+            self.risk_ratio_ci = (np.exp(np.log(rr) - zalpha * self.risk_ratio_se),
+                                  np.exp(np.log(rr) + zalpha * self.risk_ratio_se))
 
     def summary(self, decimal=3):
         """Prints a summary of the results for the doubly robust estimator. Confidence intervals are only available for
@@ -411,22 +470,29 @@ class AIPTW:
         fmt = 'Outcome:          {:<15} No. Missing Outcome:  {:<20}'
         print(fmt.format(self.outcome, np.sum(self.df[self.outcome].isnull())))
 
-        fmt = 'g-Model:          {:<15} Q-model:              {:<20}'
-        e = 'Logistic'
-        if self._continuous_outcome:
-            y = self._continuous_type
+        fmt = 'g-Model:          {:<15} Missing Model:        {:<20}'
+        if self._exp_model_custom:
+            e = 'User-specified'
         else:
-            y = 'Logistic'
-
-        print(fmt.format(e, y))
-
-        fmt = 'Missing model:    {:<15}'
-        if self._fit_missing_:
+            e = 'Logistic'
+        if self._miss_model_custom and self._fit_missing_:
+            m = 'User-specified'
+        elif self._fit_missing_:
             m = 'Logistic'
         else:
             m = 'None'
 
-        print(fmt.format(m))
+        print(fmt.format(e, m))
+
+        fmt = 'Q-Model:          {:<15}'
+        if self._out_model_custom:
+            y = 'User-specified'
+        elif self._continuous_outcome:
+            y = self._continuous_type
+        else:
+            y = 'Logistic'
+
+        print(fmt.format(y))
 
         print('======================================================================')
 
@@ -448,7 +514,12 @@ class AIPTW:
                 print(str(round(100 * (1 - self.alpha), 1)) + '% two-sided CI: -')
             print('----------------------------------------------------------------------')
             print('Risk Ratio:        ', round(float(self.risk_ratio), decimal))
-            print(str(round(100 * (1 - self.alpha), 1)) + '% two-sided CI: -')
+            if self._weight_ is None:
+                print(str(round(100 * (1 - self.alpha), 1)) + '% two-sided CI: (' +
+                      str(round(self.risk_ratio_ci[0], decimal)), ',',
+                      str(round(self.risk_ratio_ci[1], decimal)) + ')')
+            else:
+                print(str(round(100 * (1 - self.alpha), 1)) + '% two-sided CI: -')
 
         print('======================================================================')
 
