@@ -207,7 +207,10 @@ class MonteCarloGFormula:
                                      family=linkdist).fit()
 
         if print_results:
+            print('==============================================================================')
+            print('Predict-Exposure Model')
             print(self.exp_model.summary())
+            print('==============================================================================')
         self._exposure_model_fit = True
 
     def outcome_model(self, model, restriction=None, print_results=True):
@@ -242,8 +245,10 @@ class MonteCarloGFormula:
             self.out_model = smf.glm(self.outcome + ' ~ ' + model, g, freq_weights=g[self._weights],
                                      family=linkdist).fit()
         if print_results:
+            print('==============================================================================')
+            print('Outcome Model')
             print(self.out_model.summary())
-
+            print('==============================================================================')
         self._outcome_model_fit = True
 
     def censoring_model(self, model, restriction=None, print_results=True):
@@ -273,8 +278,10 @@ class MonteCarloGFormula:
             self.cens_model = smf.glm('__uncensored__ ~ ' + model, g, freq_weights=g[self._weights],
                                       family=linkdist).fit()
         if print_results:
+            print('==============================================================================')
+            print('Censoring Model')
             print(self.cens_model.summary())
-
+            print('==============================================================================')
         self._censor_model_fit = True
 
     def add_covariate_model(self, label, covariate, model, restriction=None, recode=None, var_type='binary',
@@ -338,7 +345,10 @@ class MonteCarloGFormula:
 
         f = m.fit()
         if print_results:
+            print('==============================================================================')
+            print('Covariate (' + str(covariate) + ') Model')
             print(f.summary())
+            print('==============================================================================')
 
         # Adding to lists, it is used to predict variables later on for the time-varying...
         self._covariate_models.append(f)
@@ -488,9 +498,9 @@ class MonteCarloGFormula:
         except TypeError:  # gets around pandas <0.22 error
             gs = pd.concat(mc_simulated_data, ignore_index=True)
 
-        self.predicted_outcomes = gs[
-            ['uid_g_zepid', self.exposure, self.outcome, self.time_in, self.time_out] + self._covariate].sort_values(
-            by=['uid_g_zepid', self.time_in]).reset_index(drop=True)
+        cols_to_keep = ['uid_g_zepid', self.idvar, self.exposure, self.outcome, self.time_in,
+                        self.time_out] + self._covariate
+        self.predicted_outcomes = gs[cols_to_keep].sort_values(by=['uid_g_zepid', self.time_in]).reset_index(drop=True)
 
     @staticmethod
     def _predict(df, model, variable):
@@ -623,13 +633,12 @@ class IterativeCondGFormula:
         # Check same number of treatments and outcomes
         if len(exposures) != len(outcomes):
             raise ValueError("The number of exposures must equal the number of outcomes")
-
         self.exposure = exposures
+
         # Checking that outcome is binary
         for o in outcomes:
             if not df[o].dropna().value_counts().index.isin([0, 1]).all():
                 raise ValueError('Only binary outcomes are currently implemented')
-
         self.outcome = outcomes
 
         # Checking for recurrent outcomes. Recurrent are not currently supported
@@ -714,14 +723,14 @@ class IterativeCondGFormula:
             # 2.1) Fit the model to the observed data
             if self.outcome[::-1].index(d) == 0:
                 fm = smf.glm(d + ' ~ ' + m, df, family=linkdist).fit()  # GLM
-                if self._printseqregresults:
-                    print(fm.summary())
             else:
                 df[d] = np.where(df[prior_predict].isna(), df[d], df[prior_predict])
-
                 fm = smf.glm(d + ' ~ ' + m, df, family=linkdist).fit()  # GLM
-                if self._printseqregresults:
-                    print(fm.summary())
+            if self._printseqregresults:
+                print('==============================================================================')
+                print('Sequential Outcome Model')
+                print(fm.summary())
+                print('==============================================================================')
 
             # 2.2) Generating predictions
             tf = df.copy()
