@@ -4,11 +4,83 @@ import numpy as np
 import pandas as pd
 import statsmodels.api as sm
 import statsmodels.formula.api as smf
+from scipy.stats import logistic
 from scipy.stats.kde import gaussian_kde
 from statsmodels.stats.weightstats import DescrStatsW
 import matplotlib.pyplot as plt
 
-from zepid.calc import probability_to_odds
+from zepid.calc import probability_to_odds, probability_bounds
+
+
+def check_input_data(data, exposure, outcome, estimator, drop_censoring, drop_missing, binary_exposure_only):
+    """Background function used by the various estimators to check the input data for possible issues or
+    inconsistencies with the format expected by the estimator.
+
+    Parameters
+    ----------
+    data : DataFrame
+        Input dataframe to be processed
+    exposure : str
+        Exposure/treatment column name
+    outcome : str
+        Outcome column
+    estimator : str
+        Name of the input estimator. Reads back the estimator name in the warning system
+    drop_censoring : bool
+        Whether to drop censored observations. Some estimators do not handle censoring (so this is set to True)
+    drop_missing : bool
+        Argument currently does nothing. Will be utilized in the future (when some estimators support missing data
+        aside from censoring)
+    binary_exposure_only : bool
+        Argument for whether only binary exposure / treatments are supported. This is currently True for most
+        estimators
+
+    Returns
+    -------
+    formatted DataFrame, boolean for missing data flag
+    """
+    # Handling missing data and censoring
+    if drop_censoring:  # Censoring is not always supported by all estimators
+        valid_obs = data.dropna(subset=[d for d in data.columns]).shape[0]
+        if valid_obs != data.shape[0]:
+            warnings.warn("There is missing data in the dataset. By default, " + str(estimator) +
+                          " will drop all missing data (including missing outcome data). " + str(estimator) +
+                          " will fit " + str(valid_obs) + ' of ' + str(data.shape[0]) + ' observations', UserWarning)
+            data = data.copy().dropna().reset_index()
+        else:
+            data = data.copy().reset_index()
+        miss_flag = False
+        data['__missing_indicator__'] = 1
+    else:
+        valid_obs = data.dropna(subset=[d for d in data.columns if d != outcome]).shape[0]
+        # Checking for other missing data
+        if valid_obs != data.shape[0]:
+            warnings.warn("There is missing data that is not the outcome in the data set. " + str(estimator) +
+                          " will drop all missing data that is not missing outcome data. " + str(estimator) +
+                          " will fit " + str(valid_obs) + " of " + str(data.shape[0]) + " observations", UserWarning)
+            data = data.copy().dropna(subset=[d for d in data.columns if d != outcome]).reset_index()
+        else:
+            data = data.copy().reset_index()
+        # Checking for censored data
+        if valid_obs != data.dropna(subset=[outcome]).shape[0]:
+            miss_flag = True
+            data['__missing_indicator__'] = np.where(data[outcome].isna(), 0, 1)
+        else:
+            miss_flag = False
+            data['__missing_indicator__'] = 1
+
+    # Checking for only binary exposures
+    if binary_exposure_only:
+        if not data[exposure].value_counts().index.isin([0, 1]).all():
+            raise ValueError(str(estimator) + " only supports binary exposures currently")
+
+    # Check outcome data type
+    if data[outcome].value_counts().index.isin([0, 1]).all():
+        continuous = False
+    else:
+        continuous = True
+
+    return data, miss_flag, continuous
 
 
 def propensity_score(df, model, weights=None, print_results=True):
@@ -44,10 +116,10 @@ def propensity_score(df, model, weights=None, print_results=True):
         log = smf.glm(model, df, freq_weights=df[weights], family=f).fit()
 
     if print_results:
-        print('\n----------------------------------------------------------------')
-        print('MODEL: ' + model)
-        print('-----------------------------------------------------------------')
+        print('==============================================================================')
+        print('Propensity Score Model')
         print(log.summary())
+        print('==============================================================================')
     return log
 
 
@@ -63,8 +135,11 @@ def exposure_machine_learner(xdata, ydata, ml_model, print_results=True):
                         "covers both sklearn and supylearner. If there is a predictive model you would "
                         "like to use, please open an issue at https://github.com/pzivich/zepid and I "
                         "can work on adding support")
-    if print_results and hasattr(fm, 'summarize'):  # SuPyLearner has a nice summarize function
-        fm.summarize()
+    if print_results and hasattr(fm, 'summary'):  # SuPyLearner has a nice summarize function
+        print('==============================================================================')
+        print('Propensity Score Model')
+        fm.summary()
+        print('==============================================================================')
 
     # Generating predictions
     if hasattr(fm, 'predict_proba'):
@@ -75,8 +150,7 @@ def exposure_machine_learner(xdata, ydata, ml_model, print_results=True):
         else:
             return g[:, 1]
     elif hasattr(fm, 'predict'):
-        g = fm.predict(xdata)
-        return g
+        return fm.predict(xdata)
     else:
         raise ValueError("Currently custom_model must have 'predict' or 'predict_proba' attribute")
 
@@ -93,8 +167,11 @@ def outcome_machine_learner(xdata, ydata, all_a, none_a, ml_model, continuous, p
                         "covers both sklearn and supylearner. If there is a predictive model you would "
                         "like to use, please open an issue at https://github.com/pzivich/zepid and I "
                         "can work on adding support")
-    if print_results and hasattr(fm, 'summarize'):  # Nice summarize option from SuPyLearner
-        fm.summarize()
+    if print_results and hasattr(fm, 'summary'):  # Nice summarize option from SuPyLearner
+        print('==============================================================================')
+        print('Outcome Model')
+        fm.summary()
+        print('==============================================================================')
 
     # Generating predictions
     if continuous:
@@ -112,7 +189,7 @@ def outcome_machine_learner(xdata, ydata, all_a, none_a, ml_model, continuous, p
             if (qa1.ndim == 1) and (qa0.ndim == 1):
                 return qa1, qa0
             else:
-                return qa1[:,1], qa0[:,1]
+                return qa1[:, 1], qa0[:, 1]
         elif hasattr(fm, 'predict'):
             qa1 = fm.predict(all_a)
             qa0 = fm.predict(none_a)
@@ -133,8 +210,11 @@ def stochastic_outcome_machine_learner(xdata, ydata, ml_model, continuous, print
                         "covers both sklearn and supylearner. If there is a predictive model you would "
                         "like to use, please open an issue at https://github.com/pzivich/zepid and I "
                         "can work on adding support")
-    if print_results and hasattr(fm, 'summarize'):  # Nice summarize option from SuPyLearner
-        fm.summarize()
+    if print_results and hasattr(fm, 'summary'):  # Nice summarize option from SuPyLearner
+        print('==============================================================================')
+        print('Outcome Model')
+        fm.summary()
+        print('==============================================================================')
 
     # Generating predictions
     if continuous:
@@ -195,8 +275,11 @@ def missing_machine_learner(xdata, mdata, all_a, none_a, ml_model, print_results
                         "covers both sklearn and supylearner. If there is a predictive model you would "
                         "like to use, please open an issue at https://github.com/pzivich/zepid and I "
                         "can work on adding support")
-    if print_results and hasattr(fm, 'summarize'):  # SuPyLearner has a nice summarize function
-        fm.summarize()
+    if print_results and hasattr(fm, 'summary'):  # SuPyLearner has a nice summarize function
+        print('==============================================================================')
+        print('Censoring Model')
+        fm.summary()
+        print('==============================================================================')
 
     # Generating predictions
     if hasattr(fm, 'predict_proba'):
@@ -214,41 +297,6 @@ def missing_machine_learner(xdata, mdata, all_a, none_a, ml_model, print_results
         raise ValueError("Currently custom_model must have 'predict' or 'predict_proba' attribute")
 
 
-def _bounding_(v, bounds):
-    """Creates bounding for g-bounds in models
-
-    Parameters
-    ----------
-    v:
-        -Values to be bounded
-    bounds:
-        -Percentile thresholds for bounds
-    """
-    if type(bounds) is float:  # Symmetric bounding
-        if bounds < 0 or bounds > 1:
-            raise ValueError('Bound value must be between (0, 1)')
-        v = np.where(v < bounds, bounds, v)
-        v = np.where(v > 1-bounds, 1-bounds, v)
-    elif type(bounds) is str:  # Catching string inputs
-        raise ValueError('Bounds must either be a float between (0, 1), or a collection of floats between (0, 1)')
-    elif type(bounds) is int:  # Catching string inputs
-        raise ValueError('Bounds must either be a float between (0, 1), or a collection of floats between (0, 1)')
-    else:  # Asymmetric bounds
-        if bounds[0] > bounds[1]:
-            raise ValueError('Bound thresholds must be listed in ascending order')
-        if len(bounds) > 2:
-            warnings.warn('It looks like your specified bounds is more than two floats. Only the first two '
-                          'specified bounds are used by the bound statement. So only ' +
-                          str(bounds[0:2]) + ' will be used', UserWarning)
-        if type(bounds[0]) is str or type(bounds[1]) is str:
-            raise ValueError('Bounds must be floats between (0, 1)')
-        if (bounds[0] < 0 or bounds[1] > 1) or (bounds[0] < 0 or bounds[1] > 1):
-            raise ValueError('Both bound values must be between (0, 1)')
-        v = np.where(v < bounds[0], bounds[0], v)
-        v = np.where(v > bounds[1], bounds[1], v)
-    return v
-
-
 def iptw_calculator(df, treatment, model_denom, model_numer, weight, stabilized, standardize, bound, print_results):
     """Background function to calculate inverse probability of treatment weights. Used by `IPTW`, `AIPTW`, `IPSW`,
     `AIPSW`
@@ -269,8 +317,8 @@ def iptw_calculator(df, treatment, model_denom, model_numer, weight, stabilized,
 
     # Bounding predicted probabilities if requested
     if bound:
-        d = _bounding_(d, bounds=bound)
-        n = _bounding_(n, bounds=bound)
+        d = probability_bounds(d, bounds=bound)
+        n = probability_bounds(n, bounds=bound)
 
     # Calculating weights
     if stabilized:  # Stabilized weights
@@ -693,3 +741,57 @@ def stochastic_check_conditional(df, conditional):
     if np.any(a > 1):
         warnings.warn("It looks like your conditional categories are NOT exclusive. For appropriate estimation, "
                       "the conditions that designate each category should be exclusive", UserWarning)
+
+
+def aipw_calculator(y, a, py_a, py_n, pa1, pa0, difference=True, weights=None, splits=None, continuous=False):
+    """Function to calculate AIPW estimates. Called by AIPTW, SingleCrossfitAIPTW, and DoubleCrossfitAIPTW
+    """
+    # Point estimate calculation
+    y1 = np.where(a == 1, (y - py_a*(1 - pa1)) / pa1, py_a)
+    y0 = np.where(a == 0, (y - py_n*(1 - pa0)) / pa0, py_n)
+
+    # Warning system if values are out of range
+    if not continuous:
+        if np.mean(y1) > 1 or np.mean(y1) < 0:
+            warnings.warn("The estimated probability for all-exposed is out of the bounds (less than zero or greater "
+                          "than 1). This may indicate positivity issues resulting from extreme weights, too small of a "
+                          "sample size, or too flexible of models. Try setting the optional `bound` argument. If using "
+                          "DoubleCrossfitAIPTW, try SingleCrossfitAIPTW or the TMLE estimators instead.", UserWarning)
+        if np.mean(y0) > 1 or np.mean(y0) < 0:
+            warnings.warn("The estimated probability for none-exposed is out of the bounds (less than zero or greater "
+                          "than 1). This may indicate positivity issues resulting from extreme weights, too small of a "
+                          "sample size, or too flexible of models. Try setting the optional `bound` argument. If using "
+                          "DoubleCrossfitAIPTW, try SingleCrossfitAIPTW or the TMLE estimators instead.", UserWarning)
+
+    # Calculating ACE as a difference
+    if difference:
+        if weights is None:
+            estimate = np.nanmean(y1 - y0)
+            if splits is None:
+                var = np.nanvar((y1 - y0) - estimate, ddof=1) / y.shape[0]
+            else:
+                var_rd = []
+                for i in set(splits):
+                    y1s = y1[splits == i]
+                    y0s = y0[splits == i]
+                    var_rd.append(np.var((y1s - y0s) - estimate, ddof=1))
+                var = np.mean(var_rd) / y.shape[0]
+        else:
+            estimate = DescrStatsW(y1, weights=weights).mean - DescrStatsW(y0, weights=weights).mean
+            var = np.nan
+
+    # Calculating ACE as a ratio
+    else:
+        if weights is None:
+            estimate = np.nanmean(y1) / np.nanmean(y0)
+            if estimate < 0:
+                warnings.warn("lower than 0", UserWarning)
+            py_o = a*py_a + (1-a)*py_n
+            ic = ((a*(y-py_o)) / (np.mean(py_a)*pa1) + (py_a - np.mean(py_a)) -
+                  ((1-a)*(y-py_o)) / (np.mean(py_n)*pa0) + (py_n - np.mean(py_n)))
+            var = np.nanvar(ic, ddof=1) / y.shape[0]
+        else:
+            estimate = DescrStatsW(y1, weights=weights).mean / DescrStatsW(y0, weights=weights).mean
+            var = np.nan
+
+    return estimate, var
