@@ -4,4 +4,5 @@ from .graphics import (functional_form_plot,
                        spaghetti_plot,
                        roc,
                        dynamic_risk_plot,
-                       labbe_plot)
+                       labbe_plot,
+                       zipper_plot)
